local LCCC = LibCodesCommonCode
local LMAC = LibMultiAccountCollectibles
local LEJ = LibExtendedJournal
local EventCollectibles = EventCollectibles


--------------------------------------------------------------------------------
-- Extended Journal
--------------------------------------------------------------------------------

local TAB_NAME = "EventCollectibles"
local FRAME = EventCollectiblesFrame
local DATA_TYPE = 1
local SORT_TYPE = 1

local Initialized = 0
local Dirtiness = 0
local ContextMenuItems = { }

function EventCollectibles.InitializeBrowser( )
	LEJ.RegisterTab(TAB_NAME, {
		name = SI_EVENTCOLLECTIBLES_TITLE,
		order = 400,
		iconPrefix = "/esoui/art/treeicons/achievements_indexicon_events_",
		control = FRAME,
		binding = "EVENTCOLLECTIBLES",
		slashCommands = { "/eventcollectibles" },
		callbackShow = function( )
			EventCollectibles.LazyInitializeBrowser()
			EventCollectibles.RefreshBrowser(true)
		end,
	})
end

function EventCollectibles.LazyInitializeBrowser( )
	if (Initialized == 0) then
		Initialized = 1

		-- Instantiate the browser
		EventCollectibles.list = EventCollectiblesList:New(FRAME, ContextMenuItems)

		-- Listen for changes
		local refresh = function( )
			Dirtiness = 1
			EventCollectibles.RefreshBrowser()
		end

		if (LMAC) then
			LMAC.RegisterForCallback(EventCollectibles.name, LMAC.EVENT_COLLECTION_UPDATED, refresh)
		else
			EVENT_MANAGER:RegisterForEvent(EventCollectibles.name, EVENT_COLLECTIBLES_UNLOCK_STATE_CHANGED, refresh)
		end

		Initialized = 2
	end
end

function EventCollectibles.RefreshBrowser( noActiveCheck )
	if (Initialized > 1 and Dirtiness > 0 and (noActiveCheck or LEJ.IsTabActive(TAB_NAME))) then
		EventCollectibles.list:RefreshData()
		Dirtiness = 0
	end
end


--------------------------------------------------------------------------------
-- Register Context Menu
--------------------------------------------------------------------------------

function EventCollectibles.RegisterContextMenuItem( func )
	table.insert(ContextMenuItems, func)
end

EventCollectibles.RegisterContextMenuItem(function( data )
	return SI_ITEM_ACTION_LINK_TO_CHAT, function( )
		ZO_LinkHandler_InsertLink(data.itemLink)
	end
end)


--------------------------------------------------------------------------------
-- LibMultiAccountCollectibles Support
--------------------------------------------------------------------------------

local SelectedServer, SelectedAccount
local IsCollectibleOwned = IsCollectibleOwnedByDefId
if (LMAC) then
	IsCollectibleOwned = function(...) return LMAC.IsCollectibleOwnedByAccount(SelectedServer, SelectedAccount, ...) end
	SelectedServer = LCCC.GetServerName()
end


--------------------------------------------------------------------------------
-- Local Utilities
--------------------------------------------------------------------------------

local function CountFragments( fragments )
	if (#fragments > 1) then
		local collected = 0
		for i = 2, #fragments do
			if (IsCollectibleOwned(fragments[i])) then
				collected = collected + 1
			end
		end
		return collected, #fragments - 1
	end
end


--------------------------------------------------------------------------------
-- EventCollectiblesList
--------------------------------------------------------------------------------

EventCollectiblesList = ExtendedJournalSortFilterList:Subclass()
local EventCollectiblesList = EventCollectiblesList

function EventCollectiblesList:Setup( )
	ZO_ScrollList_AddDataType(self.list, DATA_TYPE, "EventCollectiblesListRow", 30, function(...) self:SetupItemRow(...) end)
	ZO_ScrollList_EnableHighlight(self.list, "ZO_ThinListHighlight")
	self:SetAlternateRowBackgrounds(true)

	self.masterList = { }

	local sortKeys = {
		["name"]     = { caseInsensitive = true },
		["status"]   = { isNumeric = true, tiebreaker = "source", tieBreakerSortOrder = ZO_SORT_ORDER_UP },
		["category"] = { caseInsensitive = true, tiebreaker = "name", tieBreakerSortOrder = ZO_SORT_ORDER_UP },
		["source"]   = { caseInsensitive = true, tiebreaker = "category", tieBreakerSortOrder = ZO_SORT_ORDER_UP },
	}

	self.currentSortKey = "source"
	self.currentSortOrder = ZO_SORT_ORDER_UP
	self.sortHeaderGroup:SelectAndResetSortForKey(self.currentSortKey)
	self.sortFunction = function( listEntry1, listEntry2 )
		return ZO_TableOrderingFunction(listEntry1.data, listEntry2.data, self.currentSortKey, sortKeys, self.currentSortOrder)
	end

	self.filterDrop = ZO_ComboBox_ObjectFromContainer(self.frame:GetNamedChild("FilterDrop"))
	self:InitializeComboBox(self.filterDrop, { list = EventCollectibles.data, key = 1 }, EventCollectibles.vars.filterId)

	self.searchBox = self.frame:GetNamedChild("SearchFieldBox")
	self.searchBox:SetHandler("OnTextChanged", function() self:RefreshFilters() end)
	self.search = self:InitializeSearch(SORT_TYPE)

	if (LMAC) then
		local servers = LMAC.GetServerAndAccountList(true)

		if (#servers > 1 or #servers[1].accounts > 1) then
			local control = self.frame:GetNamedChild("AccountDrop")
			control:GetNamedChild("Caption"):SetText(GetString(SI_LEJ_ACCOUNT))
			control:SetHidden(false)
			self.accountDrop = ZO_ComboBox_ObjectFromContainer(control)

			if (#servers > 1) then
				local control = self.frame:GetNamedChild("ServerDrop")
				control:GetNamedChild("Caption"):SetText(GetString(SI_LEJ_SERVER))
				control:SetHidden(false)
				self.serverDrop = ZO_ComboBox_ObjectFromContainer(control)
				self:InitializeComboBox(self.serverDrop, { list = servers, key = "server" }, nil, true, function( comboBox, entryText, entry, selectionChanged )
					SelectedServer = entryText
					self:RefreshAccountList()
				end)
			else
				self:RefreshAccountList()
			end
		end
	end

	self:RefreshData()
end

function EventCollectiblesList:BuildMasterList( )
	self.masterList = { }
	for i, source in ipairs(EventCollectibles.data) do
		for j = 2, #source do
			for _, id in ipairs(source[j]) do
				local status, fragmentsKnown, fragmentsTotal
				if (type(id) == "table") then
					fragmentsKnown, fragmentsTotal = CountFragments(id)
					id = id[1]
				end

				local name, _, _, _, unlocked, _, _, categoryType = GetCollectibleInfo(id)
				if (LMAC) then unlocked = IsCollectibleOwned(id) end

				if (unlocked) then
					status = 2
				elseif (fragmentsTotal) then
					status = fragmentsKnown / fragmentsTotal
				else
					status = 0
				end

				table.insert(self.masterList, {
					type = SORT_TYPE,
					id = id,
					name = zo_strformat(SI_TOOLTIP_ITEM_NAME, name),
					status = status,
					fragmentsKnown = fragmentsKnown,
					fragmentsTotal = fragmentsTotal,
					category = GetString("SI_COLLECTIBLECATEGORYTYPE", categoryType),
					source = source[1],
					sourceId = i,
					itemLink = string.format("|H1:collectible:%d|h|h", id),
				})
			end
		end
	end
end

function EventCollectiblesList:FilterScrollList( )
	local scrollData = ZO_ScrollList_GetDataList(self.list)
	ZO_ClearNumericallyIndexedTable(scrollData)

	local filterId = self.filterDrop:GetSelectedItemData().id
	EventCollectibles.vars.filterId = filterId

	local searchInput = self.searchBox:GetText()

	local collected = 0

	for _, data in ipairs(self.masterList) do
		if ( (filterId == 1 or filterId == data.sourceId) and
		     (searchInput == "" or self.search:IsMatch(searchInput, data)) ) then
			table.insert(scrollData, ZO_ScrollList_CreateDataEntry(DATA_TYPE, data))
			if (data.status == 2) then
				collected = collected + 1
			end
		end
	end

	self.frame:GetNamedChild("CollectedCount"):SetText((#scrollData > 0) and string.format(GetString(SI_EVENTCOLLECTIBLES_COLLECTED_COUNT), collected, #scrollData, 100 * collected / #scrollData) or "")
end

function EventCollectiblesList:SetupItemRow( control, data )
	local cell

	cell = control:GetNamedChild("Name")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(data.name)
	control:GetNamedChild("Icon"):SetTexture(GetCollectibleIcon(data.id))

	cell = control:GetNamedChild("Status")
	cell.nonRecolorable = true
	if (data.status == 2) then
		cell:SetColor(0, 1, 0, 1)
		cell:SetText(GetString(SI_YES))
	else
		if (data.status == 0) then
			cell:SetColor(1, 0, 0, 1)
		else
			cell:SetColor(1, 1, 0, 1)
		end
		if (data.fragmentsTotal) then
			cell:SetText(string.format("%d / %d", data.fragmentsKnown, data.fragmentsTotal))
		else
			cell:SetText(GetString(SI_NO))
		end
	end

	cell = control:GetNamedChild("Category")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(data.category)

	cell = control:GetNamedChild("Source")
	cell.normalColor = ZO_DEFAULT_TEXT
	cell:SetText(data.source)

	self:SetupRow(control, data)
end

function EventCollectiblesList:ProcessItemEntry( stringSearch, data, searchTerm, cache )
	if ( zo_plainstrfind(data.name:lower(), searchTerm) or
	     (EventCollectibles.vars.filterId == 1 and zo_plainstrfind(data.source:lower(), searchTerm)) ) then
		return true
	end

	return false
end

function EventCollectiblesList:RefreshAccountList( )
	local accounts
	for _, server in ipairs(LMAC.GetServerAndAccountList(true)) do
		if (SelectedServer == server.server or not accounts) then
			accounts = server.accounts
		end
	end

	-- Try to keep the same account selected when changing servers
	local initialIndex
	for i, account in ipairs(accounts) do
		if (SelectedAccount == account) then
			initialIndex = i
		end
	end

	self:InitializeComboBox(self.accountDrop, { list = accounts }, initialIndex, true, function( comboBox, entryText, entry, selectionChanged )
		SelectedAccount = entryText
		Dirtiness = 1
		EventCollectibles.RefreshBrowser()
	end)
end


--------------------------------------------------------------------------------
-- XML Handlers
--------------------------------------------------------------------------------

local Tooltip = ItemTooltip

function EventCollectiblesListRow_OnMouseEnter( control )
	local data = ZO_ScrollList_GetData(control)
	EventCollectibles.list:Row_OnMouseEnter(control)

	Tooltip = LEJ.ItemTooltip({ collectibleId = data.id })
	if (LMAC and LMAC.AddTooltipExtension) then
		LMAC.AddTooltipExtension(Tooltip, data.id, SelectedServer)
	end
end

function EventCollectiblesListRow_OnMouseExit( control )
	EventCollectibles.list:Row_OnMouseExit(control)

	ClearTooltip(Tooltip)
end

function EventCollectiblesListRow_OnMouseUp( ... )
	EventCollectibles.list:Row_OnMouseUp(...)
end
